package de.kif.backend.util

import de.kif.backend.database.Connection
import de.kif.backend.repository.*
import de.kif.common.Message
import de.kif.common.RepositoryType
import de.kif.common.model.*
import kotlinx.serialization.Serializable

@Serializable
data class Backup(
    val posts: List<Post> = emptyList(),
    val rooms: List<Room> = emptyList(),
    val schedules: List<Schedule> = emptyList(),
    val tracks: List<Track> = emptyList(),
    val users: List<User> = emptyList(),
    val workGroups: List<WorkGroup> = emptyList()
) {
    companion object {
        suspend fun backup(vararg repositories: RepositoryType): String {
            var backup = Backup()

            val repositorySet = repositories.toMutableSet()

            if (RepositoryType.SCHEDULE in repositorySet) {
                repositorySet += RepositoryType.ROOM
                repositorySet += RepositoryType.WORK_GROUP
            }

            if (RepositoryType.WORK_GROUP in repositorySet) {
                repositorySet += RepositoryType.TRACK
            }

            for (repository in repositorySet) {
                backup = when (repository) {
                    RepositoryType.ROOM -> backup.copy(rooms = RoomRepository.all())
                    RepositoryType.SCHEDULE -> backup.copy(schedules = ScheduleRepository.all())
                    RepositoryType.TRACK -> backup.copy(tracks = TrackRepository.all())
                    RepositoryType.USER -> backup.copy(users = UserRepository.all())
                    RepositoryType.WORK_GROUP -> backup.copy(workGroups = WorkGroupRepository.all())
                    RepositoryType.POST -> backup.copy(posts = PostRepository.all())
                }
            }

            return Message.json.stringify(serializer(), backup)
        }

        @Suppress("UNUSED_VARIABLE")
        suspend fun import(data: String) {
            val backup = Message.json.parse(serializer(), data)

            backup.users.forEach { UserRepository.create(it) }
            backup.posts.forEach { PostRepository.create(it) }

            backup.rooms.forEach { RoomRepository.create(it) }
            val roomMap = RoomRepository.all().associateWith { it.id!! }

            backup.tracks.forEach { TrackRepository.create(it) }
            val trackMap =TrackRepository.all().associateWith { it.id!! }

            backup.workGroups.forEach {
                var workGroup = it
                val track = workGroup.track
                if (track != null) {
                    workGroup = workGroup.copy(track = track.copy(id = trackMap[track] ?: return@forEach))
                }

                WorkGroupRepository.create(workGroup)
            }
            val workGroupMap = WorkGroupRepository.all().associateWith { it.id!! }

            backup.schedules.forEach {
                ScheduleRepository.create(
                    it.copy(
                        room = it.room.copy(id = roomMap[it.room] ?: return@forEach),
                        workGroup = it.workGroup.copy(id = workGroupMap[it.workGroup] ?: return@forEach)
                    )
                )
            }
        }

        suspend fun restore(data: String) {
            Connection.reset()

            import(data)
        }
    }
}
